import { finalize } from 'rxjs/operators';
export class Cycler {
    constructor() {
        this._destructors = [];
        this._transientDestructors = {};
    }
    dispose() {
        for (let i = 0; i < this._destructors.length; i++) {
            const destructor = this._destructors[i];
            destructor();
        }
        this._destructors = [];
    }
    ngOnDestroy() {
        this.dispose();
    }
    manage(destructor) {
        this._destructors.push(destructor);
        return {
            dispose: () => {
                const destructorIndex = this._destructors.indexOf(destructor);
                if (destructorIndex > 0) {
                    destructor();
                    this._destructors.splice(destructorIndex, 1);
                }
            }
        };
    }
    manageTransient(code, destructor) {
        if (this._transientDestructors[code]) {
            this._transientDestructors[code].dispose();
        }
        const result = this.manage(destructor);
        this._transientDestructors[code] = result;
        return result;
    }
    manageSubscription(subscription) {
        return this.manage(() => subscription.unsubscribe());
    }
    manageTransientSubscription(code, subscription) {
        return this.manageTransient(code, () => {
            subscription.unsubscribe();
        });
    }
    manageSubscriptions(...subscriptions) {
        for (const subscription of subscriptions) {
            this.manageSubscription(subscription);
        }
    }
    manageDisposable(disposable) {
        this.manage(() => disposable.dispose());
    }
    managedSubscribe(observable, next) {
        let destructor = null;
        const subscription = observable
            .pipe(finalize(() => {
            if (destructor !== null) {
                this._destructors = this._destructors.filter(d => d !== destructor);
            }
        }))
            .subscribe(next);
        destructor = () => subscription.unsubscribe();
        return this.manage(destructor);
    }
}
//# sourceMappingURL=cycler.js.map